{#
    basic/localtoc.html
    ~~~~~~~~~~~~~~~~~~~

    Sphinx sidebar template: local table of contents.

    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
#}
<div>
<h4><a href="{{ pathto(master_doc)|e }}">{{ _('Sections') }}</a></h4>
{{ toc }}
</div>
